// ****************** MUST INCLUDE LINES BELOW*****************************
#include <lib6302.h>
// Init library and set pwm freq, DAC resolution, ADC resolution, and teensy 3.6-3.2 bit rate.
// PWM frequency affects DAC resolution, higher frequency, lower resolution.
// For 8-bit PWM: Teensy 3.6-234375, Teensy 3.2-187500!!!1
// Divide by 2,4,8 or 16 for 9,10,11, or 12 bits.
// WARNING 12 bit pwm is in audio range for young listeners.
#define MYSERIAL Serial
lib6302 my6302(234375/8,12,16,&Serial);
// Set up the analog-to-digital library, with a buffersize
#define sampleBufN 500
LIB6302_ADC(sampleBufN+100);  // ADC sample buffer of 100+sampleBufN
// ****************** MUST INCLUDE LINES ABOVE*******1**********************

// Constants for Users to Modify 
// ************************************************************
#define angleOffset 0.02  // Adjust if measured angle is not zero when horizontal. 
#define togglePeriod 2.5 // seconds between +/- toggling of desired value.
#define highTheta 0.1 // Toggle angle in radians high (0.5 is max)
#define lowTheta -0.1 // Toggle angle in radians low (-0.5 is min)
#define nomCmdL 0.5 // Adjust to zero steady-state error
#define nomCmdR 0.5 // Adjust to zero steady-state error
#define mBack 2
#define plotIncrement 10
#define controllerUpdatePeriod 1000 // Update period in microseconds

// ************************************************************************
// End of Constants and code for Users to Modify 
// SEE END OF FILE FOR USER-ALTERABLE CONTROL LOOP!!

// *******SETTING UP THE MONITOR DISPLAY (SLIDERS AND PLOTS). *************
// Sliders
String sliders = "S~Pause~0~1~S~Kp~0~10~S~Kd~0~5~Freq~0.1~20";
#define PA 0
#define KP 1
#define KD 2
#define FREQ 3

// Plots: USE SPACES BETWEEN LABELS FOR MULTIPLOTS!!
String measDes= "P~Theta_d Theta_m~-1~1~500~2~";
String monPlots = measDes;
#define PLOT_D 0
#define PLOT_M 1

// Configuration string with surrounding limits.
String config_message = sliders + monPlots;

// Control loop setup
// ************************************************************************

// Motor PWM and Sensor defines.
elapsedMicros loopTimer = 0;  // Loop runs at least 10xsecond

float mDeltaT = 0.0;
float deltaT = 0.0;
float headRoom;

uint32_t loopCntr = 0;

void setup() {
  MYSERIAL.begin(1000000);
  my6302.setup(config_message, togglePeriod);
  my6302.startPWM(true); // start parallel drive pwm, A+B in sync.
  my6302.setupADC(16,true);
  loopTimer = 0;
  loopCntr = 0;
  deltaT = float(controllerUpdatePeriod)*1.0e-6;
  mDeltaT = mBack*deltaT;
  headRoom = controllerUpdatePeriod;
}

#define zeroParamThresh 0.005
#define scaleParam 1.005
float analog2param(int pinA) {
  float retval = my6302.adcGetAvg(pinA) - zeroParamThresh;
  retval = scaleParam * max(retval, 0.0);
  return(retval);
}

//********************** Main USER ALTERABLE Control Loop **********************
//**************************************************************************
int loopCounter = 0;
float pastError[mBack+1];  // Stores past errors for deriv approx

void loop() {  

  // Initialize: check for inputs, flip toggle, etc.
  my6302.startloop();

  // Get parameters.
  float pauseVal= my6302.getSlider(PA);
  float Kp = my6302.getSlider(KP);
  float Kd = my6302.getSlider(KD);
  float Freq = my6302.getSlider(FREQ); // Frequency in Hertz (cycles per second).
  float periodFracDt = deltaT*Freq;  // deltaT*Freq = period fraction id One Sample
  
  // Headroom is faction of period left until next sensor read.
  float headRoomThis = float(controllerUpdatePeriod - int(loopTimer));
  headRoom = headRoomThis;
  
  // Wait exactly one deltaT since last sensor read.
  while (int(loopTimer) < controllerUpdatePeriod) {};
  loopTimer = 0;
  
  // Read the angle sensor
  float meas = 2*PI*(0.5 - my6302.adcGetAvg(A1) + angleOffset);
  

  // Compute fraction of sine wave period
  float prds  = loopCntr * periodFracDt; // Periods since loopCntr resest 
  long wholePrds = (long) prds; // whole periods since loopCntr reset.
  float periodFrac = prds - wholePrds; // Fraction of period since last whole one.

  // Reset loopCntr when near end of a whole period, or when too large
  if((loopCntr>1000000000) || ((loopCntr>1000000)&&(periodFrac<0.01*periodFracDt))) 
              loopCntr = 0;
   else loopCntr  += 1;

  // If Freq = 0, toggle desired value, otherwise output sine.
  float desired = highTheta;
  if (Freq == 0) {  // Toggle
    float flip = my6302.getFlip();
    if(flip > 0) desired = lowTheta;
  } else {  // Sine output
    desired *= sin(2*PI*periodFrac);
  }
  
  // Compute the errors
  float error = (desired - meas);
  
  // Save backward angles for delta/deltaT calculation.
  for (int i = mBack; i > 0; i--) pastError[i] = pastError[i-1];
  pastError[0] = error;

  // Compute (error - oldError)/deltaT.
  float delDt = (error -pastError[mBack])/mDeltaT;
  
  // Compute feedback contribution to motor cmd = Kp*Err+Kd*D(Err)/Dt
  float dCmd = Kp*error + Kd*delDt; 

  // Write motor command Hbridges, return clipped value.
  // FIX THIS!  Is dCmd used correctly?
  float cmdCR = my6302.hbridgeWrite(nomCmdL + dCmd, pwmB2); // B1->B2 flips motorndir
  float cmdCL = my6302.hbridgeWrite(nomCmdR + dCmd, pwmB1_T); // B1_T->B2_T flips motor dir
  
  if(loopCounter <  plotIncrement) loopCounter++;
  else {
    loopCounter = 1;
    if(pauseVal == 0) {
      // Send data to monitor.
      float loopStatus[2];
      loopStatus[PLOT_D] = desired;
      loopStatus[PLOT_M] = meas;
      my6302.sendStatus(2,loopStatus);  
    }
  }
}




  


  
